/****************************************************************************
    This file is part of TIImageTool.

    TIImageTool is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TIImageTool is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TIImageTool.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright 2011 Michael Zapf
    www.mizapf.de
    
****************************************************************************/
package ti.image;

import javax.swing.*;
import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.awt.Cursor;

import ti.assm.Assembler;
import ti.util.Utilities;
import ti.files.FormatException;

import ti.files.Element;
import ti.files.File;
import ti.files.Volume;
import ti.files.Directory;

public class DisassembleAction extends Activity {

	public String getMenuName() {
		return "Disassemble TMS99xx...";
	}
	
	public String getActionName() {
		return "DISASS";
	}
	
	public void go() {
		String sText = "no content";
		DirectoryView dvCurrent = imagetool.getSelectedView();
		Directory dirCurrent = dvCurrent.getDirectory();
		DirectoryPanel dp = dvCurrent.getPanel();
		
		m_parent.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		for (Element selected : dvCurrent.getSelectedEntries()) {
			if (selected instanceof File) {
				try {
					byte[] content = null;
					if (((File)selected).isProgram()) content = ((File)selected).getRawContent();
					else content = ((File)selected).getRecordContent();
					
					DisassParamDialog disparm = new DisassParamDialog(dvCurrent.getFrame(), content, false);
					disparm.setType(((File)selected).isProgram());
					disparm.createGui(imagetool.boldFont);
					String sHash = Utilities.getHash(content);
					disparm.setParams(imagetool.loadDisassParams(TIImageTool.ASMPREFIX, sHash));
					disparm.setVisible(true);
					if (disparm.confirmed()) {			
						imagetool.saveDisassParams(TIImageTool.ASMPREFIX, sHash, disparm.getStartAddress(), disparm.getOffset(), disparm.getLength(), disparm.getHint(), false);

						if (((File)selected).isProgram() && disparm.symbolic()) {
							byte[] toc = createTocFromDump(content, disparm.getOffset(), disparm.getStartAddress());
							Assembler tms9900 = new Assembler(toc);
							sText = tms9900.disassembleTaggedObjectCode(disparm.getOffset(), disparm.getStartAddress(), disparm.getLength(), disparm.getHint(), disparm.showDataLoc());						
						}
						else {
							Assembler tms9900 = new Assembler(content);
							if (((File)selected).isProgram())
								sText = tms9900.disassembleMemoryImage(disparm.getOffset(), disparm.getStartAddress(), disparm.getLength(), disparm.getHint(), disparm.showDataLoc());
							else 
								sText = tms9900.disassembleTaggedObjectCode(disparm.getOffset(), disparm.getStartAddress(), disparm.getLength(), disparm.getHint(), disparm.showDataLoc());
						}
							
						imagetool.showTextContent("Disassembled " + selected.getName(), sText);
					}
				}
				catch (IOException iox) {
					JOptionPane.showMessageDialog(dvCurrent.getFrame(), iox.getClass().getName(), "Error opening file", JOptionPane.ERROR_MESSAGE); 
				}
				catch (FormatException fx) {
					JOptionPane.showMessageDialog(dvCurrent.getFrame(), "Parameter error: " + fx.getMessage(), "Disassembler error", JOptionPane.ERROR_MESSAGE); 
				}
				catch (ImageException ix) {
					JOptionPane.showMessageDialog(dvCurrent.getFrame(), "Image error: " + ix.getMessage(), "Read error", JOptionPane.ERROR_MESSAGE); 
				}
			}
		}
		m_parent.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
	
	/** Creates a tagged object code from a dump so that it will be disassembled
		with the features of the TocDisassembler.
		
		TODO: Autoselect all binaries
	*/
	private byte[] createTocFromDump(byte[] content, int offset, int startAddr) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		baos.write((byte)0x01);
		baos.write((byte)0x00);
		baos.write((byte)0x00);
		for (int i=0; i < 8; i++) baos.write(' ');
		baos.write('9');
		baos.write((startAddr >> 8)&0xff);
		baos.write(startAddr&0xff);
		
		offset = offset & 0xfffffffe;
		for (int i=offset; i < content.length; i++) {
			if ((i & 1)==0) baos.write('B');
			baos.write(content[i]);
		}
		baos.write(':');
		return baos.toByteArray();
	}
}
