/****************************************************************************
    This file is part of TIImageTool.

    TIImageTool is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TIImageTool is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TIImageTool.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright 2011 Michael Zapf
    www.mizapf.de
    
****************************************************************************/
package ti.image;

import javax.swing.JOptionPane;
import java.io.IOException;
import ti.files.FormatException;
import ti.files.Element;
import ti.files.File;
import ti.files.Volume;
import ti.files.Directory;
import java.awt.Cursor;
import java.io.EOFException;
import ti.util.Utilities;

public class ViewTextAction extends Activity {

	public String getMenuName() {
		return "View as text";
	}
	
	public String getActionName() {
		return "VIEWTEXT";
	}
	
	public void go() {
		DirectoryView dvCurrent = imagetool.getSelectedView();
		Directory dirCurrent = dvCurrent.getDirectory();
		Volume vol = dvCurrent.getVolume();

		String sText = "no content";
		m_parent.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		// Caution: Only open text files as text 
		for (Element selected : dvCurrent.getSelectedEntries()) {
			if (selected instanceof File) {
				try {
					String sContent = ((File)selected).getTextContent();
					if (Utilities.checkForText(sContent)==false) {
						JOptionPane.showMessageDialog(dvCurrent.getFrame(), "File contains lots of unprintable characters; may be corrupt.", "Read error", JOptionPane.ERROR_MESSAGE); 					
					}
					else {
						sContent = Utilities.sanitizeText(sContent);
						imagetool.showTextContent(selected.getName(), sContent);  
					}
				}
				catch (EOFException eofx) {
					JOptionPane.showMessageDialog(dvCurrent.getFrame(), "Error: " + eofx.getMessage(), "Read error", JOptionPane.ERROR_MESSAGE); 					
				}
				catch (IOException iox) {
					JOptionPane.showMessageDialog(dvCurrent.getFrame(), "Error reading file: " + iox.getClass().getName(), "Read error", JOptionPane.ERROR_MESSAGE); 
				}
				catch (ImageException ix) {
					JOptionPane.showMessageDialog(dvCurrent.getFrame(), "Image error: " + ix.getMessage(), "Read error", JOptionPane.ERROR_MESSAGE); 
				}
				catch (FormatException fx) {
					JOptionPane.showMessageDialog(dvCurrent.getFrame(), fx.toString(), "Read error", JOptionPane.ERROR_MESSAGE); 
				}
			}
		}
		m_parent.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
}
