package ti.util;

public class Waitstates {
	
	private final static int PAD = 1;
	private final static int SRAM = 2;
	private final static int DRAM = 3;
	
	public static int wait;
	
	public static void main(String[] arg) {
		if (arg.length<1) {
			System.err.println("java Waitstates [sram|dram] [video] [extra] <operation_string>");
			System.err.println("  sram/dram: where commands are read from (default pad)");
			System.err.println("  video: video waitstates");
			System.err.println("  extra: extra waitstates");
			return;
		}
		
		int command = PAD;
		boolean video = false;
		boolean extra = false;
		String sSequence = "";
		
		for (int i=0; i < arg.length; i++) {
			if (arg[i].equalsIgnoreCase("sram")) command=SRAM;
			else {
				if (arg[i].equalsIgnoreCase("dram")) command=DRAM;
				else {
					if (arg[i].equalsIgnoreCase("video")) video = true; 
					else {
						if (arg[i].equalsIgnoreCase("extra")) extra = true;
						else sSequence = arg[i];
					}
				}
			}
		}
				
		System.out.println("|....:....|....:....|....:....|....:....|....:....|....:....|");
		System.out.println(calculateWaitstates(sSequence, command, video, extra));
	}
	
	public static String calculateWaitstates(String sInput, int command, boolean videowait, boolean extrawait) {
		StringBuilder sb = new StringBuilder();
		for (int i=0; i < sInput.length(); i++) {
			char ch = sInput.charAt(i);
// ----------------------------------------------
/*
   Buchstaben = Befehl
   < = Externes Lesen (1 Byte) SRAM
   > = Externes Schreiben (1 Byte) SRAM
   ( = Externes Lesen (1 Byte) DRAM
   ) = Externes Schreiben (1 Byte) DRAM
   « = VDP lesen
   » = VDP schreiben
   ← = internes Lesen (1 Wort)
   → = internes Lesen (1 Wort)
   @ = Argument lesen (siehe Befehl)
*/
			final char PAD_RD = '←';
			final char PAD_WT = '→';
			final char SRAM_RD = '<';
			final char SRAM_WT = '>';
			final char DRAM_RD = '(';
			final char DRAM_WT = ')';
			final char VIDEO_RD = '«';
			final char VIDEO_WT = '»';
			final char DEV_RD = '{';
			final char DEV_WT = '}';			

			switch(ch) {
			case PAD_RD:
				access(sb,ch);
				break;
			case PAD_WT:
				access(sb,ch);
				break;
			case SRAM_RD:
				if (extrawait) minimumWait(2);
				doWait(sb);
				access(sb,ch);
				break;
			case SRAM_WT:
				wait--;
				if (extrawait) minimumWait(2);
				access(sb,ch);
				doWait(sb);
				break;
			case DRAM_RD:
				minimumWait(1);
				if (extrawait) minimumWait(2);
				doWait(sb);
				access(sb,ch);
				break;
			case DRAM_WT:
				wait--;
				minimumWait(1);
				if (extrawait) minimumWait(2);
				access(sb,ch);
				doWait(sb);
				break;
			case DEV_RD:   // Test
				minimumWait(1);
				if (extrawait) minimumWait(2);
				doWait(sb);
				access(sb,ch);
				break;
			case DEV_WT:    // Test
				wait--;
				minimumWait(1);
				if (extrawait) minimumWait(2);
				access(sb,ch);
				doWait(sb);
				break;
			case VIDEO_RD:
				wait=0;
				minimumWait(1);
				if (extrawait) minimumWait(2);
				doWait(sb);
				access(sb,ch);
				if (videowait) wait=15;
				break;
			case VIDEO_WT:
				wait=0;
				minimumWait(1);
				if (extrawait) minimumWait(2);
				access(sb,ch);
				doWait(sb);
				if (videowait) wait=16;
				break;
			default:
				// Befehl
				if (command==SRAM) {
					if (extrawait) minimumWait(2);
					doWait(sb);
				}
				if (command==DRAM) {
					minimumWait(1);
					if (extrawait) minimumWait(2);
					doWait(sb);
				}
				access(sb,ch);
				break;
			}
			if (wait > 0) wait--;
			
// ----------------------------------------------
		}
		return sb.toString();
	}
	
	private static void minimumWait(int add) {
		if (wait < add) wait = add;
	}
	
	private static void doWait(StringBuilder sb) {
		for (int j=0; j < wait; j++) sb.append('·'); 
		wait = 0;
	}
	
	private static void access(StringBuilder sb, char ch) {
		sb.append(ch);
	}
}

/*
			switch(ch) {
			case '←':
				sb.append(ch);
				break;
			case '→':
				sb.append(ch);
				break;
			case '<':
				if (extra && wait<2) wait=2;
				for (int j=0; j < wait; j++) sb.append('·'); wait = 0;
				sb.append(ch);
				break;
			case '>':
				wait--;
				if (extra && wait<2) wait=2;
				sb.append(ch);
				for (int j=0; j < wait; j++) sb.append('·'); wait = 0;
				break;
			case '(':
				if (wait<1) wait=1;
				if (extra && wait<2) wait=2;
				for (int j=0; j < wait; j++) sb.append('·'); wait = 0;
				sb.append(ch);
				break;
			case ')':
				wait--;
				if (wait<1) wait=1;
				sb.append(ch);
				for (int j=0; j < wait; j++) sb.append('·'); wait = 0;
				break;
			case '«':
				wait=1;
				if (extra) wait=2;
				for (int j=0; j < wait; j++) sb.append('·'); wait = 0;
				sb.append(ch);
				if (video) wait=15;
				break;
			case '»':
				sb.append(ch);
				wait=1;
				if (extra) wait=2;
				for (int j=0; j < wait; j++) sb.append('·'); wait = 0;
				if (video) wait=16;
				break;
			default:
				// Befehl
				if (command!=PAD) {
					if (extra && wait<2) wait=2;
					for (int j=0; j < wait; j++) sb.append('·'); wait = 0;
				}
				sb.append(ch);
				break;
			}
			if (wait > 0) wait--;
*/
